/*
 lexer.c - functions and variables shared between several lexer implementations 
 * Copyright (c) Warren Toomey, under the GPL3 license.
 * Copyright (c) Red Hat, Inc - author Florian Festi <ffesti@redhat.com>
 * Much of this file comes from cslang.l, a part of the CSlang program
 * which is Copyright (C) 1995 Tudor Hulubei under the GPL.
 *
 * $Revision: 1.4 $
 */

#include "liblexer.h"

void py_restart(FILE * input_file);
int py_lex(void);
void c_restart(FILE * input_file);
int c_lex(void);
void j_restart(FILE * input_file);
int j_lex(void);
void hex_restart(FILE * input_file);
int hex_lex(void);

void reset_lexer(void);
void output_filename(char *name);

int inside_comment = 0;
int indent = 0;

#define endswith(suff)	!strcmp(suff, file + strlen(file) - strlen(suff))

void tokenize(char *file)
{
  FILE *zin;

  /* fast check for the most common suffixes
   * This code comes from Eric Raymond's comparator 1.0 source code,
   * which is (c) Eric Raymond under the GPL.
   */

  if (endswith(".o") || endswith("~") || endswith(".pyc"))
    return;
  else if (strstr(file, "CVS") || strstr(file, "RCS") ||
	   strstr(file, "SCCS") || strstr(file, ".git") ||
	   strstr(file, ".hg"))
    return;

  zin = fopen(file, "r");
  if (zin == NULL)
    return;

  reset_lexer();

  if (endswith(".c") || endswith(".h") ||
      endswith(".s") ||
      endswith(".C") || endswith(".H") ||
      endswith(".C++") || endswith(".Cpp")) {
    output_filename(file);
    c_restart(zin);
    c_lex();
  } else if (endswith(".py") || endswith(".PY")) {
    output_filename(file);
    py_restart(zin);
    py_lex();
  } else if (endswith(".java") || endswith(".JAVA")) {
    output_filename(file);
    j_restart(zin);
    j_lex();
  } else if (endswith(".hex") || endswith(".HEX")) {
    output_filename(file);
    hex_restart(zin);
    hex_lex();
  }
  fclose(zin);
}

#undef endswith

void myputc(char ch, FILE * f)
{
  if (inside_comment && (ch != LINE))
    return;
  putc(ch, f);
}

void myputindent(size_t depth, FILE * f)
{
  size_t i;
  /* printf("INDENT %i %i", depth, indent); */
  if (inside_comment)
    return;
  for (i = indent; i < depth; i++)
    myputc(INDENT, f);
  for (i = indent; i > depth; i--)
    myputc(OUTDENT, f);
  indent = depth;
}

/* CCITT CRC-16 hash:
 * returns a 16-bit number
 * for the input string.
 */
unsigned int get_hashval(char *t)
{
  unsigned int crc = 0;

  while (*t != '\0') {
    crc = (unsigned char) (crc >> 8) | (crc << 8);
    crc ^= *t;
    crc ^= (unsigned char) (crc & 0xff) >> 4;
    crc ^= (crc << 8) << 4;
    crc ^= ((crc & 0xff) << 4) << 1;
    t++;
  }
  /* printf("Got %d for %s\n", h, oldt); */
  return (crc & 0xffff);
}


/* This function called at the beginning of each input file */
void reset_lexer(void)
{
  inside_comment = 0;
  indent = 0;
}


/* Given a STRINGLIT, CHARCONST, INTVAL, IDENTIFIER or LABEL,
 * output the token followed by a 16-bit hash of the value.
 */
void myputtokhash(char ch, char *text, FILE * f)
{
  unsigned int hash;
  char *pos;

  /* Output the token to start with */
  if (inside_comment)
    return;
  putc(ch, f);

  hash = get_hashval(text);
  /* printf("Hash %04x %s\n", hash, text); */
  putc(hash / 256, f);
  putc(hash & 255, f);
  for (pos = strchr(text, '\n'); pos != NULL; pos = strchr(++pos, '\n')) {
    /* printf("NEWLINE in COMMENT: %s", text); */
    putc(LINE, f);
  }
}
